@@ -40,7 +40,7 @@ public:
     void removeMainWindow(Konsole::MainWindow *mainWindow);
 
     virtual void createWidgetsForMainWindow(Konsole::MainWindow *mainWindow) = 0;
-    virtual void activeViewChanged(Konsole::SessionController *controller) = 0;
+    virtual void activeViewChanged(Konsole::SessionController *controller, Konsole::MainWindow *mainWindow) = 0;
 
     virtual QList<QAction *> menuBarActions(Konsole::MainWindow *mainWindow) const
     {
