@@ -335,6 +335,16 @@ public:
         return _bidiEnabled;
     }
 
+    bool semanticUpDown() const
+    {
+        return _semanticUpDown;
+    }
+
+    int semanticHints() const
+    {
+        return _semanticHints;
+    }
+
     ColorSchemeWallpaper::Ptr wallpaper() const
     {
         return _wallpaper;
@@ -354,6 +364,11 @@ public:
     // returns 0 - not selecting, 1 - pending selection (button pressed but no movement yet), 2 - selecting
     int selectionState() const;
 
+    bool trimTrailingWhitespace() const
+    {
+        return _trimTrailingSpaces;
+    }
+
     Qt::Edge droppedEdge() const
     {
         return _overlayEdge;
@@ -394,7 +409,7 @@ public Q_SLOTS:
     void copyToX11Selection();
 
     /** Copies the selected text to the system clipboard. */
-    void copyToClipboard();
+    void copyToClipboard(Screen::DecodingOptions options = Screen::PlainText);
 
     /**
      * Pastes the content of the clipboard into the display.
@@ -444,6 +459,8 @@ public Q_SLOTS:
     // Used for requestPrint
     void printScreen();
 
+    void notificationClicked(const QString &xdgActivationToken);
+
 Q_SIGNALS:
     void requestToggleExpansion();
     void requestMoveToNewTab(TerminalDisplay *display);
@@ -482,29 +499,13 @@ Q_SIGNALS:
      */
     void overrideShortcutCheck(QKeyEvent *keyEvent, bool &override);
 
+    void activationRequest(const QString &xdgActivationToken);
     void sendStringToEmu(const QByteArray &local8BitString);
 
     void compositeFocusChanged(bool focused);
 
     void peekPrimaryRequested(bool doPeek);
 
-    void drawContents(Character *image,
-                      QPainter &paint,
-                      const QRect &rect,
-                      bool printerFriendly,
-                      int imageSize,
-                      bool bidiEnabled,
-                      QVector<LineProperty> lineProperties);
-    void drawCurrentResultRect(QPainter &painter, const QRect &searchResultRect);
-
-    void highlightScrolledLines(QPainter &painter, bool isTimerActive, QRect rect);
-    QRegion highlightScrolledLinesRegion(TerminalScrollBar *scrollBar);
-
-    void drawBackground(QPainter &painter, const QRect &rect, const QColor &backgroundColor, bool useOpacitySetting);
-    void
-    drawCharacters(QPainter &painter, const QRect &rect, const QString &text, Character style, const QColor &characterColor, const LineProperty lineProperty);
-    void drawInputMethodPreeditString(QPainter &painter, const QRect &rect, TerminalDisplay::InputMethodData &inputMethodData, Character *image);
-
 protected:
     // events
     bool event(QEvent *event) override;
@@ -636,83 +637,83 @@ private:
 
     bool _bellMasked;
 
-    QVBoxLayout *_verticalLayout;
+    QVBoxLayout *_verticalLayout = nullptr;
 
-    int _lines; // the number of lines that can be displayed in the widget
-    int _columns; // the number of columns that can be displayed in the widget
+    int _lines = 1; // the number of lines that can be displayed in the widget
+    int _columns = 1; // the number of columns that can be displayed in the widget
 
     // Character line and character column as per a previous call to
     // getCharacterPosition() in mouseMoveEvent().
-    int _prevCharacterLine;
-    int _prevCharacterColumn;
+    int _prevCharacterLine = -1;
+    int _prevCharacterColumn = -1;
 
-    int _usedLines; // the number of lines that are actually being used, this will be less
+    int _usedLines = 1; // the number of lines that are actually being used, this will be less
     // than 'lines' if the character image provided with setImage() is smaller
     // than the maximum image size which can be displayed
 
-    int _usedColumns; // the number of columns that are actually being used, this will be less
+    int _usedColumns = 1; // the number of columns that are actually being used, this will be less
     // than 'columns' if the character image provided with setImage() is smaller
     // than the maximum image size which can be displayed
 
     QRect _contentRect;
-    Character *_image; // [lines][columns]
+    Character *_image = nullptr; // [lines][columns]
     // only the area [usedLines][usedColumns] in the image contains valid data
 
-    int _imageSize;
+    int _imageSize = 0;
     QVector<LineProperty> _lineProperties;
 
     QColor _colorTable[TABLE_COLORS];
 
-    uint _randomSeed;
+    uint _randomSeed = 0;
 
-    bool _resizing;
-    bool _showTerminalSizeHint;
-    bool _bidiEnabled;
-    bool _usesMouseTracking;
-    bool _allowMouseTracking;
-    bool _bracketedPasteMode;
+    bool _resizing = false;
+    bool _showTerminalSizeHint = true;
+    bool _bidiEnabled = false;
+    bool _usesMouseTracking = false;
+    bool _allowMouseTracking = true;
+    bool _bracketedPasteMode = false;
 
     QPoint _iPntSel; // initial selection point
     QPoint _pntSel; // current selection point
     QPoint _tripleSelBegin; // help avoid flicker
-    int _actSel; // selection state
-    bool _wordSelectionMode;
-    bool _lineSelectionMode;
-    bool _preserveLineBreaks;
-    bool _columnSelectionMode;
+    int _actSel = 0; // selection state
+    bool _wordSelectionMode = false;
+    bool _lineSelectionMode = false;
+    bool _preserveLineBreaks = true;
+    bool _columnSelectionMode = false;
 
-    bool _autoCopySelectedText;
-    bool _copyTextAsHTML;
-    Enum::MiddleClickPasteModeEnum _middleClickPasteMode;
+    bool _autoCopySelectedText = false;
+    bool _copyTextAsHTML = true;
+    Enum::MiddleClickPasteModeEnum _middleClickPasteMode = Enum::PasteFromX11Selection;
 
     QString _wordCharacters;
-    TerminalBell _bell;
+    TerminalBell _bell{Enum::NotifyBell};
 
-    bool _allowBlinkingText; // allow text to blink
-    bool _allowBlinkingCursor; // allow cursor to blink
-    bool _textBlinking; // text is blinking, hide it when drawing
-    bool _cursorBlinking; // cursor is blinking, hide it when drawing
-    bool _hasTextBlinker; // has characters to blink
-    QTimer *_blinkTextTimer;
-    QTimer *_blinkCursorTimer;
+    bool _allowBlinkingText = true; // allow text to blink
+    bool _allowBlinkingCursor = false; // allow cursor to blink
+    bool _textBlinking = false; // text is blinking, hide it when drawing
+    bool _cursorBlinking = false; // cursor is blinking, hide it when drawing
+    bool _hasTextBlinker = false; // has characters to blink
+    QTimer *_blinkTextTimer = nullptr;
+    QTimer *_blinkCursorTimer = nullptr;
 
-    bool _openLinksByDirectClick; // Open URL and hosts by single mouse click
+    bool _openLinksByDirectClick = false; // Open URL and hosts by single mouse click
 
-    bool _ctrlRequiredForDrag; // require Ctrl key for drag selected text
-    bool _dropUrlsAsText; // always paste URLs as text without showing copy/move menu
+    bool _ctrlRequiredForDrag = true; // require Ctrl key for drag selected text
+    bool _dropUrlsAsText = false; // always paste URLs as text without showing copy/move menu
 
-    Enum::TripleClickModeEnum _tripleClickMode;
-    bool _possibleTripleClick; // is set in mouseDoubleClickEvent and deleted
+    Enum::TripleClickModeEnum _tripleClickMode = Enum::SelectWholeLine;
+    bool _possibleTripleClick = false; // is set in mouseDoubleClickEvent and deleted
     // after QApplication::doubleClickInterval() delay
 
-    QLabel *_resizeWidget;
-    QTimer *_resizeTimer;
+    QLabel *_resizeWidget = nullptr;
+    QTimer *_resizeTimer = nullptr;
 
-    bool _flowControlWarningEnabled;
+    bool _flowControlWarningEnabled = false;
 
     // widgets related to the warning message that appears when the user presses Ctrl+S to suspend
     // terminal output - informing them what has happened and how to resume output
-    KMessageWidget *_outputSuspendedMessageWidget;
+    KMessageWidget *_outputSuspendedMessageWidget = nullptr;
 
     QSize _size;
 
@@ -721,10 +722,10 @@ private:
 
     // list of filters currently applied to the display.  used for links and
     // search highlight
-    TerminalImageFilterChain *_filterChain;
-    bool _filterUpdateRequired;
+    TerminalImageFilterChain *_filterChain = nullptr;
+    bool _filterUpdateRequired = true;
 
-    Enum::CursorShapeEnum _cursorShape;
+    Enum::CursorShapeEnum _cursorShape = Enum::BlockCursor;
 
     InputMethodData _inputMethodData;
 
@@ -734,44 +735,48 @@ private:
     // the duration of the size hint in milliseconds
     static const int SIZE_HINT_DURATION = 1000;
 
-    SessionController *_sessionController;
+    SessionController *_sessionController = nullptr;
 
-    bool _trimLeadingSpaces; // trim leading spaces in selected text
-    bool _trimTrailingSpaces; // trim trailing spaces in selected text
-    bool _mouseWheelZoom; // enable mouse wheel zooming or not
+    bool _trimLeadingSpaces = false; // trim leading spaces in selected text
+    bool _trimTrailingSpaces = false; // trim trailing spaces in selected text
+    bool _mouseWheelZoom = false; // enable mouse wheel zooming or not
 
-    int _margin; // the contents margin
-    bool _centerContents; // center the contents between margins
+    int _margin = 1; // the contents margin
+    bool _centerContents = false; // center the contents between margins
 
-    KMessageWidget *_readOnlyMessageWidget; // Message shown at the top when read-only mode gets activated
+    KMessageWidget *_readOnlyMessageWidget = nullptr; // Message shown at the top when read-only mode gets activated
 
     // Needed to know whether the mode really changed between update calls
-    bool _readOnly;
+    bool _readOnly = false;
 
-    bool _dimWhenInactive;
+    bool _dimWhenInactive = false;
     int _dimValue;
 
     ScrollState _scrollWheelState;
-    IncrementalSearchBar *_searchBar;
-    TerminalHeaderBar *_headerBar;
+    IncrementalSearchBar *_searchBar = nullptr;
+    TerminalHeaderBar *_headerBar = nullptr;
     QRect _searchResultRect;
     friend class TerminalDisplayAccessible;
 
-    bool _drawOverlay;
+    bool _drawOverlay = false;
     Qt::Edge _overlayEdge;
 
     bool _hasCompositeFocus;
-    bool _displayVerticalLine;
-    int _displayVerticalLineAtChar;
+    bool _displayVerticalLine = false;
+    int _displayVerticalLineAtChar = 80;
 
     QKeySequence _peekPrimaryShortcut;
 
-    TerminalPainter *_terminalPainter;
-    TerminalScrollBar *_scrollBar;
-    TerminalColor *_terminalColor;
+    TerminalPainter *_terminalPainter = nullptr;
+    TerminalScrollBar *_scrollBar = nullptr;
+    TerminalColor *_terminalColor = nullptr;
     std::unique_ptr<TerminalFont> _terminalFont;
 
     std::unique_ptr<KonsolePrintManager> _printManager;
+
+    int _semanticHints;
+    bool _semanticUpDown;
+    bool _semanticInputClick;
 };
 
 }
